@@ -14,6 +14,7 @@ module Werewolf.Command.Pass (
     handle,
 ) where
 
+import Control.Lens
 import Control.Monad.Except
 import Control.Monad.Extra
 import Control.Monad.Random
@@ -23,10 +24,13 @@ import Control.Monad.Writer
 import Data.Text (Text)
 
 import Game.Werewolf
-import Game.Werewolf.Command.Witch
+import Game.Werewolf.Command
+import Game.Werewolf.Command.Necromancer as Necromancer
+import Game.Werewolf.Command.Witch       as Witch
+import Game.Werewolf.Engine
+import Game.Werewolf.Message.Error
 
-import Werewolf.Game
-import Werewolf.Messages
+import Werewolf.System
 
 handle :: (MonadIO m, MonadRandom m) => Text -> Text -> m ()
 handle callerName tag = do
@@ -36,7 +40,12 @@ handle callerName tag = do
 
     game <- readGame tag
 
-    let command = passCommand callerName
+    command <- case game ^. stage of
+            NecromancersTurn    -> return $ Necromancer.passCommand callerName
+            WitchsTurn          -> return $ Witch.passCommand callerName
+            _                   -> exitWith failure
+                { messages = [playerCannotDoThatRightNowMessage callerName]
+                }
 
     result <- runExceptT . runWriterT $ execStateT (apply command >> checkStage >> checkGameOver) game
     case result of
