@@ -1,13 +1,12 @@
 {-|
 Module      : Game.Werewolf.Player
-Description : Simplistic player data structure with functions for searching, filtering and querying
+Description : Simplistic player data structure with lenses for searching, filtering and querying
               lists of players.
 Copyright   : (c) Henry J. Wylde, 2016
 License     : BSD3
 Maintainer  : public@hjwylde.com
 
-Players are quite simple in themselves. They have a 'name', 'role' and 'state'. Any complex
-behaviour is handled in "Game.Werewolf.Command" and "Game.Werewolf.Engine".
+Players are quite simple in themselves. They have a 'name', 'role' and 'state'.
 -}
 
 {-# LANGUAGE FlexibleContexts #-}
@@ -25,26 +24,30 @@ module Game.Werewolf.Player (
     newPlayer,
 
     -- ** Traversals
-    alphaWolf, beholder, crookedSenator, druid, fallenAngel, hunter, jester, lycan, orphan,
-    protector, scapegoat, seer, simpleVillager, simpleWerewolf, trueVillager, villageDrunk, witch,
-    villager, werewolf,
+    alphaWolf, beholder, crookedSenator, druid, dullahan, fallenAngel, hunter, jester, lycan,
+    medusa, necromancer, oracle, orphan, protector, saint, scapegoat, seer, simpleVillager,
+    simpleWerewolf, spitefulVillager, trueVillager, villageDrunk, witch, zombie,
+    loner, villager, werewolf,
 
-    -- | These are provided just as a bit of sugar to avoid continually writing @'traverse' .@.
+    -- | The following traversals are provided just as a bit of sugar to avoid continually writing
+    --   @'traverse' .@.
     names, roles, states,
 
-    -- | N.B., these are not legal traversals for the same reason 'filtered' isn't!
-    alphaWolves, beholders, crookedSenators, druids, fallenAngels, hunters, jesters, lycans,
-    orphans, protectors, scapegoats, seers, simpleVillagers, simpleWerewolves, trueVillagers,
-    villageDrunks, witches,
-    villagers, werewolves,
+    -- | N.B., the following traversals are not legal for the same reason 'filtered' isn't!
+    named,
+    alphaWolves, beholders, crookedSenators, druids, dullahans, fallenAngels, hunters, jesters,
+    lycans, medusas, necromancers, oracles, orphans, protectors, saints, scapegoats, seers,
+    simpleVillagers, simpleWerewolves, spitefulVillagers, trueVillagers, villageDrunks, witches,
+    zombies,
+    loners, villagers, werewolves,
     alive, dead,
 ) where
 
-import Control.Lens       hiding (isn't)
 import Control.Lens.Extra
 
 import Data.Function
-import Data.Text     (Text)
+import Data.String.Humanise
+import Data.Text            as T
 
 import Game.Werewolf.Role hiding (name)
 
@@ -67,13 +70,16 @@ makeLenses ''Player
 instance Eq Player where
     (==) = (==) `on` view name
 
+instance Humanise Player where
+    humanise = view name
+
 makePrisms ''State
 
 -- | Creates a new 'Alive' player.
 newPlayer :: Text -> Role -> Player
 newPlayer name role = Player name role Alive
 
--- | The traversal of 'Player's with a 'alphaWolfRole'.
+-- | The traversal of 'Player's with an 'alphaWolfRole'.
 --
 -- @
 -- 'alphaWolf' = 'role' . 'only' 'alphaWolfRole'
@@ -97,6 +103,14 @@ beholder = role . only beholderRole
 crookedSenator :: Traversal' Player ()
 crookedSenator = role . only crookedSenatorRole
 
+-- | The traversal of 'Player's with a 'dullahanRole'.
+--
+-- @
+-- 'dullahan' = 'role' . 'only' 'dullahanRole'
+-- @
+dullahan :: Traversal' Player ()
+dullahan = role . only dullahanRole
+
 -- | The traversal of 'Player's with a 'druidRole'.
 --
 -- @
@@ -105,7 +119,7 @@ crookedSenator = role . only crookedSenatorRole
 druid :: Traversal' Player ()
 druid = role . only druidRole
 
--- | The traversal of 'Player's with an 'fallenAngelRole'.
+-- | The traversal of 'Player's with a 'fallenAngelRole'.
 --
 -- @
 -- 'fallenAngel' = 'role' . 'only' 'fallenAngelRole'
@@ -137,7 +151,31 @@ jester = role . only jesterRole
 lycan :: Traversal' Player ()
 lycan = role . only lycanRole
 
--- | The traversal of 'Player's with a 'orphanRole'.
+-- | The traversal of 'Player's with a 'medusaRole'.
+--
+-- @
+-- 'medusa' = 'role' . 'only' 'medusaRole'
+-- @
+medusa :: Traversal' Player ()
+medusa = role . only medusaRole
+
+-- | The traversal of 'Player's with a 'necromancerRole'.
+--
+-- @
+-- 'necromancer' = 'role' . 'only' 'necromancerRole'
+-- @
+necromancer :: Traversal' Player ()
+necromancer = role . only necromancerRole
+
+-- | The traversal of 'Player's with a 'oracleRole'.
+--
+-- @
+-- 'oracle' = 'role' . 'only' 'oracleRole'
+-- @
+oracle :: Traversal' Player ()
+oracle = role . only oracleRole
+
+-- | The traversal of 'Player's with an 'orphanRole'.
 --
 -- @
 -- 'orphan' = 'role' . 'only' 'orphanRole'
@@ -153,6 +191,14 @@ orphan = role . only orphanRole
 protector :: Traversal' Player ()
 protector = role . only protectorRole
 
+-- | The traversal of 'Player's with a 'saintRole'.
+--
+-- @
+-- 'saint' = 'role' . 'only' 'saintRole'
+-- @
+saint :: Traversal' Player ()
+saint = role . only saintRole
+
 -- | The traversal of 'Player's with a 'scapegoatRole'.
 --
 -- @
@@ -185,6 +231,14 @@ simpleVillager = role . only simpleVillagerRole
 simpleWerewolf :: Traversal' Player ()
 simpleWerewolf = role . only simpleWerewolfRole
 
+-- | The traversal of 'Player's with a 'spitefulVillagerRole'.
+--
+-- @
+-- 'spitefulVillager' = 'role' . 'only' 'spitefulVillagerRole'
+-- @
+spitefulVillager :: Traversal' Player ()
+spitefulVillager = role . only spitefulVillagerRole
+
 -- | The traversal of 'Player's with a 'trueVillagerRole'.
 --
 -- @
@@ -209,6 +263,22 @@ villageDrunk = role . only villageDrunkRole
 witch :: Traversal' Player ()
 witch = role . only witchRole
 
+-- | The traversal of 'Player's with a 'zombieRole'.
+--
+-- @
+-- 'zombie' = 'role' . 'only' 'zombieRole'
+-- @
+zombie :: Traversal' Player ()
+zombie = role . only zombieRole
+
+-- | The traversal of 'Player's aligned with 'NoOne'.
+--
+-- @
+-- 'loner' = 'role' . 'allegiance' . '_NoOne'
+-- @
+loner :: Traversal' Player ()
+loner = role . allegiance . _NoOne
+
 -- | The traversal of 'Player's aligned with the 'Villagers'.
 --
 -- @
@@ -225,7 +295,7 @@ villager = role . allegiance . _Villagers
 werewolf :: Traversal' Player ()
 werewolf = role . allegiance . _Werewolves
 
--- | This 'Traversal' provides the traversal of 'Player' names.
+-- | The traversal of 'Player' names.
 --
 -- @
 -- 'names' = 'traverse' . 'name'
@@ -233,7 +303,7 @@ werewolf = role . allegiance . _Werewolves
 names :: Traversable t => Traversal' (t Player) Text
 names = traverse . name
 
--- | This 'Traversal' provides the traversal of 'Player' roles.
+-- | The traversal of 'Player' roles.
 --
 -- @
 -- 'roles' = 'traverse' . 'role'
@@ -241,7 +311,7 @@ names = traverse . name
 roles :: Traversable t => Traversal' (t Player) Role
 roles = traverse . role
 
--- | This 'Traversal' provides the traversal of 'Player' states.
+-- | The traversal of 'Player' states.
 --
 -- @
 -- 'states' = 'traverse' . 'state'
@@ -249,7 +319,15 @@ roles = traverse . role
 states :: Traversable t => Traversal' (t Player) State
 states = traverse . state
 
--- | This 'Traversal' provides the traversal of 'alphaWolf' 'Player's.
+-- | The traversal of 'Player's with the given name.
+--
+-- @
+-- 'named' name' = 'filteredBy' . 'name' name'
+-- @
+named :: Text -> Traversal' Player Player
+named name' = filteredBy name name'
+
+-- | The traversal of 'alphaWolf' 'Player's.
 --
 -- @
 -- 'alphaWolves' = 'traverse' . 'filtered' ('is' 'alphaWolf')
@@ -257,7 +335,7 @@ states = traverse . state
 alphaWolves :: Traversable t => Traversal' (t Player) Player
 alphaWolves = traverse . filtered (is alphaWolf)
 
--- | This 'Traversal' provides the traversal of 'beholder' 'Player's.
+-- | The traversal of 'beholder' 'Player's.
 --
 -- @
 -- 'beholders' = 'traverse' . 'filtered' ('is' 'beholder')
@@ -265,7 +343,7 @@ alphaWolves = traverse . filtered (is alphaWolf)
 beholders :: Traversable t => Traversal' (t Player) Player
 beholders = traverse . filtered (is beholder)
 
--- | This 'Traversal' provides the traversal of 'crookedSenator' 'Player's.
+-- | The traversal of 'crookedSenator' 'Player's.
 --
 -- @
 -- 'crookedSenators' = 'traverse' . 'filtered' ('is' 'crookedSenator')
@@ -273,7 +351,15 @@ beholders = traverse . filtered (is beholder)
 crookedSenators :: Traversable t => Traversal' (t Player) Player
 crookedSenators = traverse . filtered (is crookedSenator)
 
--- | This 'Traversal' provides the traversal of 'druid' 'Player's.
+-- | The traversal of 'dullahan' 'Player's.
+--
+-- @
+-- 'dullahans' = 'traverse' . 'filtered' ('is' 'dullahan')
+-- @
+dullahans :: Traversable t => Traversal' (t Player) Player
+dullahans = traverse . filtered (is dullahan)
+
+-- | The traversal of 'druid' 'Player's.
 --
 -- @
 -- 'druids' = 'traverse' . 'filtered' ('is' 'druid')
@@ -281,7 +367,7 @@ crookedSenators = traverse . filtered (is crookedSenator)
 druids :: Traversable t => Traversal' (t Player) Player
 druids = traverse . filtered (is druid)
 
--- | This 'Traversal' provides the traversal of 'fallenAngel' 'Player's.
+-- | The traversal of 'fallenAngel' 'Player's.
 --
 -- @
 -- 'fallenAngels' = 'traverse' . 'filtered' ('is' 'fallenAngel')
@@ -289,7 +375,7 @@ druids = traverse . filtered (is druid)
 fallenAngels :: Traversable t => Traversal' (t Player) Player
 fallenAngels = traverse . filtered (is fallenAngel)
 
--- | This 'Traversal' provides the traversal of 'hunter' 'Player's.
+-- | The traversal of 'hunter' 'Player's.
 --
 -- @
 -- 'hunters' = 'traverse' . 'filtered' ('is' 'hunter')
@@ -297,7 +383,7 @@ fallenAngels = traverse . filtered (is fallenAngel)
 hunters :: Traversable t => Traversal' (t Player) Player
 hunters = traverse . filtered (is hunter)
 
--- | This 'Traversal' provides the traversal of 'jester' 'Player's.
+-- | The traversal of 'jester' 'Player's.
 --
 -- @
 -- 'jesters' = 'traverse' . 'filtered' ('is' 'jester')
@@ -305,7 +391,7 @@ hunters = traverse . filtered (is hunter)
 jesters :: Traversable t => Traversal' (t Player) Player
 jesters = traverse . filtered (is jester)
 
--- | This 'Traversal' provides the traversal of 'lycan' 'Player's.
+-- | The traversal of 'lycan' 'Player's.
 --
 -- @
 -- 'lycans' = 'traverse' . 'filtered' ('is' 'lycan')
@@ -313,7 +399,31 @@ jesters = traverse . filtered (is jester)
 lycans :: Traversable t => Traversal' (t Player) Player
 lycans = traverse . filtered (is lycan)
 
--- | This 'Traversal' provides the traversal of 'orphan' 'Player's.
+-- | The traversal of 'medusa' 'Player's.
+--
+-- @
+-- 'medusas' = 'traverse' . 'filtered' ('is' 'medusa')
+-- @
+medusas :: Traversable t => Traversal' (t Player) Player
+medusas = traverse . filtered (is medusa)
+
+-- | The traversal of 'necromancer' 'Player's.
+--
+-- @
+-- 'necromancers' = 'traverse' . 'filtered' ('is' 'necromancer')
+-- @
+necromancers :: Traversable t => Traversal' (t Player) Player
+necromancers = traverse . filtered (is necromancer)
+
+-- | The traversal of 'oracle' 'Player's.
+--
+-- @
+-- 'oracles' = 'traverse' . 'filtered' ('is' 'oracle')
+-- @
+oracles :: Traversable t => Traversal' (t Player) Player
+oracles = traverse . filtered (is oracle)
+
+-- | The traversal of 'orphan' 'Player's.
 --
 -- @
 -- 'orphans' = 'traverse' . 'filtered' ('is' 'orphan')
@@ -321,7 +431,7 @@ lycans = traverse . filtered (is lycan)
 orphans :: Traversable t => Traversal' (t Player) Player
 orphans = traverse . filtered (is orphan)
 
--- | This 'Traversal' provides the traversal of 'protector' 'Player's.
+-- | The traversal of 'protector' 'Player's.
 --
 -- @
 -- 'protectors' = 'traverse' . 'filtered' ('is' 'protector')
@@ -329,7 +439,15 @@ orphans = traverse . filtered (is orphan)
 protectors :: Traversable t => Traversal' (t Player) Player
 protectors = traverse . filtered (is protector)
 
--- | This 'Traversal' provides the traversal of 'scapegoat' 'Player's.
+-- | The traversal of 'saint' 'Player's.
+--
+-- @
+-- 'saints' = 'traverse' . 'filtered' ('is' 'saint')
+-- @
+saints :: Traversable t => Traversal' (t Player) Player
+saints = traverse . filtered (is saint)
+
+-- | The traversal of 'scapegoat' 'Player's.
 --
 -- @
 -- 'scapegoats' = 'traverse' . 'filtered' ('is' 'scapegoat')
@@ -337,7 +455,7 @@ protectors = traverse . filtered (is protector)
 scapegoats :: Traversable t => Traversal' (t Player) Player
 scapegoats = traverse . filtered (is scapegoat)
 
--- | This 'Traversal' provides the traversal of 'seer' 'Player's.
+-- | The traversal of 'seer' 'Player's.
 --
 -- @
 -- 'seers' = 'traverse' . 'filtered' ('is' 'seer')
@@ -345,7 +463,7 @@ scapegoats = traverse . filtered (is scapegoat)
 seers :: Traversable t => Traversal' (t Player) Player
 seers = traverse . filtered (is seer)
 
--- | This 'Traversal' provides the traversal of 'simpleVillager' 'Player's.
+-- | The traversal of 'simpleVillager' 'Player's.
 --
 -- @
 -- 'simpleVillagers' = 'traverse' . 'filtered' ('is' 'simpleVillager')
@@ -353,7 +471,7 @@ seers = traverse . filtered (is seer)
 simpleVillagers :: Traversable t => Traversal' (t Player) Player
 simpleVillagers = traverse . filtered (is simpleVillager)
 
--- | This 'Traversal' provides the traversal of 'simpleWerewolf' 'Player's.
+-- | The traversal of 'simpleWerewolf' 'Player's.
 --
 -- @
 -- 'simpleWerewolves' = 'traverse' . 'filtered' ('is' 'simpleWerewolf')
@@ -361,7 +479,15 @@ simpleVillagers = traverse . filtered (is simpleVillager)
 simpleWerewolves :: Traversable t => Traversal' (t Player) Player
 simpleWerewolves = traverse . filtered (is simpleWerewolf)
 
--- | This 'Traversal' provides the traversal of 'trueVillager' 'Player's.
+-- | The traversal of 'spitefulVillager' 'Player's.
+--
+-- @
+-- 'spitefulVillagers' = 'traverse' . 'filtered' ('is' 'spitefulVillager')
+-- @
+spitefulVillagers :: Traversable t => Traversal' (t Player) Player
+spitefulVillagers = traverse . filtered (is spitefulVillager)
+
+-- | The traversal of 'trueVillager' 'Player's.
 --
 -- @
 -- 'trueVillagers' = 'traverse' . 'filtered' ('is' 'trueVillager')
@@ -369,7 +495,7 @@ simpleWerewolves = traverse . filtered (is simpleWerewolf)
 trueVillagers :: Traversable t => Traversal' (t Player) Player
 trueVillagers = traverse . filtered (is trueVillager)
 
--- | This 'Traversal' provides the traversal of 'villageDrunk' 'Player's.
+-- | The traversal of 'villageDrunk' 'Player's.
 --
 -- @
 -- 'villageDrunks' = 'traverse' . 'filtered' ('is' 'villageDrunk')
@@ -377,7 +503,7 @@ trueVillagers = traverse . filtered (is trueVillager)
 villageDrunks :: Traversable t => Traversal' (t Player) Player
 villageDrunks = traverse . filtered (is villageDrunk)
 
--- | This 'Traversal' provides the traversal of 'witch' 'Player's.
+-- | The traversal of 'witch' 'Player's.
 --
 -- @
 -- 'witches' = 'traverse' . 'filtered' ('is' 'witch')
@@ -385,7 +511,23 @@ villageDrunks = traverse . filtered (is villageDrunk)
 witches :: Traversable t => Traversal' (t Player) Player
 witches = traverse . filtered (is witch)
 
--- | This 'Traversal' provides the traversal of 'villager' 'Player's.
+-- | The traversal of 'zombie' 'Player's.
+--
+-- @
+-- 'zombies' = 'traverse' . 'filtered' ('is' 'zombie')
+-- @
+zombies :: Traversable t => Traversal' (t Player) Player
+zombies = traverse . filtered (is zombie)
+
+-- | The traversal of 'loner' 'Player's.
+--
+-- @
+-- 'loners' = 'traverse' . 'filtered' . ('is' 'loner')
+-- @
+loners :: Traversable t => Traversal' (t Player) Player
+loners = traverse . filtered (is loner)
+
+-- | The traversal of 'villager' 'Player's.
 --
 -- @
 -- 'villagers' = 'traverse' . 'filtered' ('is' 'villager')
@@ -393,7 +535,7 @@ witches = traverse . filtered (is witch)
 villagers :: Traversable t => Traversal' (t Player) Player
 villagers = traverse . filtered (is villager)
 
--- | This 'Traversal' provides the traversal of 'werewolf' 'Player's.
+-- | The traversal of 'werewolf' 'Player's.
 --
 -- @
 -- 'werewolves' = 'traverse' . 'filtered' ('is' 'werewolf')
@@ -401,7 +543,7 @@ villagers = traverse . filtered (is villager)
 werewolves :: Traversable t => Traversal' (t Player) Player
 werewolves = traverse . filtered (is werewolf)
 
--- | This 'Traversal' provides the traversal of 'Alive' 'Player's.
+-- | The traversal of 'Alive' 'Player's.
 --
 -- @
 -- 'alive' = 'filtered' ('has' $ 'state' . '_Alive')
@@ -409,7 +551,7 @@ werewolves = traverse . filtered (is werewolf)
 alive :: Traversal' Player Player
 alive = filtered (has $ state . _Alive)
 
--- | This 'Traversal' provides the traversal of 'Dead' 'Player's.
+-- | The traversal of 'Dead' 'Player's.
 --
 -- @
 -- 'dead' = 'filtered' ('has' $ 'state' . '_Dead')
